<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Kinsey Elementor Extension
 *
 * The main class that initiates and runs the plugin.
 *
 * @since 1.0.0
 */
final class Kinsey_Elementor_Extension {

	/**
	 * Minimum Elementor Version
	 *
	 * @since 1.0.0
	 *
	 * @var string Minimum Elementor version required to run the plugin.
	 */
	const MINIMUM_ELEMENTOR_VERSION = '3.5.0';

	/**
	 * Minimum PHP Version
	 *
	 * @since 1.0.0
	 *
	 * @var string Minimum PHP version required to run the plugin.
	 */
	const MINIMUM_PHP_VERSION = '7.0';

	/**
	 * Instance
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 * @static
	 *
	 * @var Kinsey_Elementor_Extension The single instance of the class.
	 */
	private static $_instance = null;

	/**
	 * Instance
	 *
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 * @static
	 *
	 * @return Kinsey_Elementor_Extension An instance of the class.
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function __construct() {
		add_action( 'plugins_loaded', array( $this, 'init' ) );
	}

	/**
	 * Initialize the plugin
	 *
	 * Load the plugin only after Elementor (and other plugins) are loaded.
	 * Checks for basic plugin requirements, if one check fail don't continue,
	 * if all check have passed load the files required to run the plugin.
	 *
	 * Fired by `plugins_loaded` action hook.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function init() {
		// Check if Elementor installed and activated
		if ( ! did_action( 'elementor/loaded' ) ) {
			add_action( 'admin_notices', array( $this, 'admin_notice_missing_main_plugin' ) );
			return;
		}

		// Check for required Elementor version
		if ( ! version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
			add_action( 'admin_notices', array( $this, 'admin_notice_minimum_elementor_version' ) );
			return;
		}

		// Check for required PHP version
		if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
			add_action( 'admin_notices', array( $this, 'admin_notice_minimum_php_version' ) );
			return;
		}

		// Add Plugin actions
		add_action( 'elementor/elements/categories_registered', array( $this, 'add_elementor_widget_categories' ) );
		add_action( 'elementor/widgets/register', array( $this, 'init_widgets' ) );

		// Add WPML's Translation Management Support
		if ( class_exists( 'SitePress' ) ) {
			add_action( 'init', array( $this, 'add_widgets_wpml_support' ) );
		}
	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have Elementor installed or activated.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_missing_main_plugin() {
		if ( isset( $_GET['activate'] ) ) {
			unset( $_GET['activate'] );
		}

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor */
			esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', 'kinsey' ),
			'<strong>' . esc_html__( 'Kinsey Elementor Extension', 'kinsey' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'kinsey' ) . '</strong>'
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required Elementor version.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_minimum_elementor_version() {
		if ( isset( $_GET['activate'] ) ) {
			unset( $_GET['activate'] );
		}

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'kinsey' ),
			'<strong>' . esc_html__( 'Kinsey Elementor Extension', 'kinsey' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'kinsey' ) . '</strong>',
			self::MINIMUM_ELEMENTOR_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required PHP version.
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function admin_notice_minimum_php_version() {
		if ( isset( $_GET['activate'] ) ) {
			unset( $_GET['activate'] );
		}

		$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'kinsey' ),
			'<strong>' . esc_html__( 'Kinsey Elementor Extension', 'kinsey' ) . '</strong>',
			'<strong>' . esc_html__( 'PHP', 'kinsey' ) . '</strong>',
			self::MINIMUM_PHP_VERSION
		);

		printf( '<div class="notice notice-error is-dismissible"><p>%1$s</p></div>', $message );
	}

	/**
	 * Register Custom Widget Categories
	 *
	 * @return void
	 */
	public function add_elementor_widget_categories( $elements_manager ) {
		$elements_manager->add_category(
			'kinsey-static',
			array(
				'title' => esc_html__( 'Kinsey Static Widgets', 'kinsey' ),
				'icon'  => 'eicon-plug',
			)
		);

		$elements_manager->add_category(
			'kinsey-dynamic',
			array(
				'title' => esc_html__( 'Kinsey Dynamic Widgets', 'kinsey' ),
				'icon'  => 'eicon-sitemap',
			)
		);
	}

	/**
	 * Include Widgets files
	 *
	 * Load widgets files
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function include_widgets_files() {
		// Custom Base Widget
		require_once __DIR__ . '/widgets/base/kinsey-widget-base.php';

		// Dynamic Widgets Require
		require_once __DIR__ . '/widgets/dynamic/portfolio-fullscreen-horizontal-scrolling.php';
		require_once __DIR__ . '/widgets/dynamic/portfolio-fullscreen-slider-backgrounds-1.php';
		require_once __DIR__ . '/widgets/dynamic/portfolio-fullscreen-slider-backgrounds-2.php';
		require_once __DIR__ . '/widgets/dynamic/portfolio-fullscreen-slider-images-1.php';
		require_once __DIR__ . '/widgets/dynamic/portfolio-halfscreen-slider-backgrounds.php';
		require_once __DIR__ . '/widgets/dynamic/portfolio-list-backgrounds.php';
		require_once __DIR__ . '/widgets/dynamic/portfolio-masonry-grid.php';
		require_once __DIR__ . '/widgets/dynamic/portfolio-slider-cards-backgrounds.php';
		require_once __DIR__ . '/widgets/dynamic/posts-list-backgrounds.php';
		require_once __DIR__ . '/widgets/dynamic/posts-masonry-grid.php';
		require_once __DIR__ . '/widgets/dynamic/services-grid.php';

		// Static Widgets Require
		require_once __DIR__ . '/widgets/static/swatch.php';
		require_once __DIR__ . '/widgets/static/button.php';
		require_once __DIR__ . '/widgets/static/coming-soon.php';
		require_once __DIR__ . '/widgets/static/content-block.php';
		require_once __DIR__ . '/widgets/static/counter.php';
		require_once __DIR__ . '/widgets/static/definitions-list.php';
		require_once __DIR__ . '/widgets/static/google-map.php';
		require_once __DIR__ . '/widgets/static/images-fullscreen-horizontal-scrolling.php';
		require_once __DIR__ . '/widgets/static/lightbox-video.php';
		require_once __DIR__ . '/widgets/static/logos-grid.php';
		require_once __DIR__ . '/widgets/static/masonry-grid-images.php';
		require_once __DIR__ . '/widgets/static/parallax-background.php';
		require_once __DIR__ . '/widgets/static/slider-images.php';
		require_once __DIR__ . '/widgets/static/slider-testimonials.php';
		require_once __DIR__ . '/widgets/static/steps-list.php';
		require_once __DIR__ . '/widgets/static/team-member.php';
	}

	/**
	 * WPML compatibility classes
	 *
	 * Include files with WPML compatability classes
	 * for widgets with repeater translatable fields
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function include_wpml_files() {
		require_once __DIR__ . '/widgets/compatibility/wpml-definitions-list.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-google-map.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-portfolio-fullscreen-horizontal-scrolling.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-portfolio-fullscreen-slider-backgrounds-1.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-portfolio-fullscreen-slider-backgrounds-2.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-portfolio-fullscreen-slider-images-1.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-portfolio-halfscreen-slider-backgrounds.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-portfolio-slider-cards-backgrounds.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-posts-list-backgrounds.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-posts-masonry-grid.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-services-grid.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-slider-testimonials.php';
		require_once __DIR__ . '/widgets/compatibility/wpml-steps-list.php';
	}

	/**
	 * Init Widgets
	 *
	 * Include widgets files and register them
	 *
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function init_widgets() {
		$this->include_widgets_files();

		// Dynamic Widgets Init
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Portfolio_Fullscreen_Horizontal_Scrolling::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Portfolio_Fullscreen_Slider_Backgrounds_1::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Portfolio_Fullscreen_Slider_Backgrounds_2::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Portfolio_Fullscreen_Slider_Images_1::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Portfolio_Halfscreen_Slider_Backgrounds::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Portfolio_List_Backgrounds::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Portfolio_Masonry_Grid::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Portfolio_Slider_Cards_Backgrounds::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Posts_List_Backgrounds::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Posts_Masonry_Grid::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Services_Grid::instance() );

		// Static Widgets Init
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Button::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Coming_Soon::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Content_Block::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Counter::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Definitions_List::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Google_Map::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Images_Fullscreen_Horizontal_Scrolling::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Lightbox_Video::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Logos_Grid::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Masonry_Grid_Images::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Parallax_Background::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Slider_Images::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Slider_Testimonials::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Steps_List::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Swatch::instance() );
		\Elementor\Plugin::instance()->widgets_manager->register( \Elementor\Kinsey_Widget_Team_Member::instance() );
	}

	/**
	 * WPML support
	 *
	 * Get widget translatable fields
	 * that will appear in WPML Translation backend
	 *
	 * @since 1.0.0
	 * @access private
	 */
	public function add_widgets_wpml_support() {
		$this->include_wpml_files();
		$this->include_widgets_files();

		// Dynamic Widgets Translation
		\Elementor\Kinsey_Widget_Portfolio_Fullscreen_Horizontal_Scrolling::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Portfolio_Fullscreen_Slider_Backgrounds_1::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Portfolio_Fullscreen_Slider_Backgrounds_2::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Portfolio_Fullscreen_Slider_Images_1::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Portfolio_Halfscreen_Slider_Backgrounds::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Portfolio_Masonry_Grid::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Portfolio_Slider_Cards_Backgrounds::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Posts_List_Backgrounds::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Posts_Masonry_Grid::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Services_Grid::instance()->add_wpml_support();

		// Static Widgets Translation
		\Elementor\Kinsey_Widget_Button::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Coming_Soon::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Content_Block::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Counter::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Definitions_List::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Google_Map::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Images_Fullscreen_Horizontal_Scrolling::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Lightbox_Video::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Masonry_Grid_Images::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Slider_Testimonials::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Steps_List::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Swatch::instance()->add_wpml_support();
		\Elementor\Kinsey_Widget_Team_Member::instance()->add_wpml_support();
	}
}
Kinsey_Elementor_Extension::instance();
